import { ILifetime } from '../Lifetime';
import { IServiceProvider } from '../ServiceProvider';
import { DependencyConstructor, DependencyOptions, IServiceCollection, Key, LifetimeConstructor, Selector, Stateful, StatefulDependencyConstructor } from './IServiceCollection';
import { MockSetup } from './mockUtils';
export declare type RecordCollection<E> = Record<Key<E>, ILifetime<unknown, E>>;
export declare class ServiceCollection<E = any> implements IServiceCollection<E> {
    private readonly lifetimes;
    constructor(other?: ServiceCollection);
    replaceSingleton<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    replaceTransient<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    replaceScoped<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    replace<T>(Lifetime: LifetimeConstructor<T, E>, dependency: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAddSingleton<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAddTransient<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAddScoped<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAdd<T>(Lifetime: LifetimeConstructor<T, E>, dependency: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addStateful<T, P>(Constructor: StatefulDependencyConstructor<T, E, P>, selector: Selector<Stateful<P, T>, E>): void;
    addSingleton<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addTransient<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addScoped<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addMultiple(Lifetime: LifetimeConstructor, dependencies: Partial<{
        [key in keyof E]: DependencyConstructor<E[key], E>;
    }>): void;
    add<T>(Lifetime: LifetimeConstructor<T, E>, dependency: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    get<T>(selector: Selector<T, E>): ILifetime<T, E> | undefined;
    remove<T>(selector: Selector<T, E>): ILifetime<T, E> | undefined;
    build(): IServiceProvider<E>;
    buildMock(mock?: MockSetup<E>): IServiceProvider<E>;
    private extractFactory;
    private cloneLifetimes;
}
