"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceCollection = void 0;
var Errors_1 = require("../Errors");
var Lifetime_1 = require("../Lifetime");
var ServiceProvider_1 = require("../ServiceProvider");
var utils_1 = require("../utils");
var mockUtils_1 = require("./mockUtils");
var ServiceCollection = /** @class */ (function () {
    function ServiceCollection(other) {
        var _a;
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.lifetimes = (_a = other === null || other === void 0 ? void 0 : other.lifetimes) !== null && _a !== void 0 ? _a : {};
    }
    Object.defineProperty(ServiceCollection.prototype, "replaceSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.replace(Lifetime_1.Singleton, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.replace(Lifetime_1.Transient, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.replace(Lifetime_1.Scoped, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replace", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependency, selector) {
            this.remove(selector);
            this.add(Lifetime, dependency, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.tryAdd(Lifetime_1.Singleton, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.tryAdd(Lifetime_1.Transient, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.tryAdd(Lifetime_1.Scoped, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAdd", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependency, selector) {
            try {
                this.add(Lifetime, dependency, selector);
            }
            catch (e) {
                if (e.type !== Errors_1.DependencyErrorType.Duplicate)
                    throw e;
            }
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addStateful", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Constructor, selector) {
            var key = (0, utils_1.extractSelector)(selector).toString();
            var last = { name: "".concat(key, "@constructor") };
            function next(scope) {
                scope[key] = scope[key] || 1;
                return scope[key]++;
            }
            return this.addTransient({
                factory: function (_, original) {
                    var _a;
                    var stateful = new ServiceProvider_1.ScopedContext(original);
                    var _b = (_a = original.lastSingleton) !== null && _a !== void 0 ? _a : {}, isSingleton = _b.isSingleton, name = _b.name;
                    var singleton = name ? "".concat(String(name), "@") : '';
                    var track = {
                        name: "".concat(singleton).concat(key, "@creator#").concat(next(original.scope)),
                        isSingleton: isSingleton,
                    };
                    return {
                        create: function (props) {
                            var inParent = original.depth;
                            var context = inParent ? original : stateful;
                            if (inParent)
                                context.enter(last);
                            context.enter(track);
                            var result = new Constructor(context.proxy, props);
                            context.leave(track);
                            if (inParent)
                                context.leave(last);
                            return result;
                        },
                    };
                },
            }, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.add(Lifetime_1.Singleton, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.add(Lifetime_1.Transient, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.add(Lifetime_1.Scoped, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addMultiple", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependencies) {
            var _this = this;
            Object.entries(dependencies).map(function (_a) {
                var key = _a[0], value = _a[1];
                return ({
                    key: key,
                    value: value,
                });
            }).forEach(function (_a) {
                var key = _a.key, value = _a.value;
                return _this.add(Lifetime, value, key);
            });
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "add", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependency, selector) {
            var name = (0, utils_1.extractSelector)(selector);
            if (name in this.lifetimes)
                throw new Errors_1.DuplicateDependencyError(name);
            this.lifetimes[name] = new Lifetime(name, this.extractFactory(dependency));
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "get", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            return this.lifetimes[(0, utils_1.extractSelector)(selector)];
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "remove", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var result = this.get(selector);
            if (result)
                delete this.lifetimes[result.name];
            return result;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "build", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return new ServiceProvider_1.ServiceProvider(this.cloneLifetimes());
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "buildMock", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (mock) {
            if (mock === void 0) { mock = {}; }
            return (0, mockUtils_1.proxyLifetimes)(this, mock);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "extractFactory", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Option) {
            return typeof Option === 'function'
                ? function (p) { return new Option(p); }
                : Option.factory;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "cloneLifetimes", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return Object.entries(this.lifetimes)
                .map(function (_a) {
                var key = _a[0], value = _a[1];
                return [key, value.clone()];
            })
                .reduce(function (acc, _a) {
                var key = _a[0], value = _a[1];
                acc[key] = value;
                return acc;
            }, {});
        }
    });
    return ServiceCollection;
}());
exports.ServiceCollection = ServiceCollection;
